from interp.grid.delaunay import dgrid as basegrid
from interp.tools import baker_exact_2D as exact_func

import numpy as np

class rect_grid(basegrid):
  def __init__(self, xres = 5, yres = 5):
    xmin = -1.0
    xmax =  1.0
    xspan = xmax - xmin
    xdel = xspan / float(xres - 1)

    ymin = -1.0
    ymay =  1.0
    yspan = ymay - ymin
    ydel = yspan / float(yres - 1)


    verts = []
    q      = []
    for x in xrange(xres):
      cur_x = xmin + (x * xdel)
      for y in xrange(yres):
        cur_y = ymin + (y * ydel)
        verts.append([cur_x, cur_y])
        q.append(exact_func((cur_x, cur_y)))
    basegrid.__init__(self, verts, q)


class random_grid(rect_grid):
  def __init__(self, num_verts = 10):
    verts = []
    q = []

    r = np.random
    appx_side_res = int(np.sqrt(num_verts))
    delta = 1.0 / float(appx_side_res)

    for x in xrange(appx_side_res + 1):
      cur_x = x * delta
      for cur_y in (0, 1):
        new_point = [cur_x, cur_y]
        verts.append(new_point)
        q.append(exact_func(new_point))

    for y in xrange(appx_side_res + 1):
      cur_y = y * delta
      for cur_x in (0, 1):
        new_point = [cur_x, cur_y]
        verts.append(new_point)
        q.append(exact_func(new_point))

    for i in xrange(num_verts):
      cur_x = r.rand()
      cur_y = r.rand()

      verts.append([cur_x, cur_y])
      q.append( exact_func( (cur_x, cur_y) ) )
    basegrid.__init__(self, verts, q)

    self.verts = np.array(self.verts)
    self.q      = np.array(self.q)
